/*---------------------------------------------------------------------------\
| Project:          LV2A Avionics Power System CAN Node (PIC18F458)
|
| File:             power.c
|
| Description:      Routines for the APS node
|
| Global Routines:  <list>
|
| Local Routines:   <list>
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Stolen from ADG code.
\---------------------------------------------------------------------------*/

//------------------------------------------------------------------------
//  #includes
//------------------------------------------------------------------------

#include <piccore.h>   // Includes for the PicCore framework

#if CORE_VERSION != 5
#error HEY! The core version has changed! Make SURE your interfaces match!
#endif

#include "power.h"   // Includes for the power node

//------------------------------------------------------------------------
// Global variable declarations
//------------------------------------------------------------------------


// Umbilical voltage global variables

uint16_t    umbilical_voltage;          // ADC reading of umbilical voltage
bit         umbilical_voltage_ready_b;  // umbilical voltage measurement ready
bit         umbilical_state_b;          // 0=low V (invalid), 1=high V (valid)

// Avionics Power System state variables

bit         avionics_power_state;       // 0 = power off, 1 = power on

// flags for when a can message is ready to be sent

bit umb_plug_status_message_flag;
bit umb_rocket_ready_message_flag;
bit umb_shore_power_message_flag;
bit avionic_cmd_message_flag;

// Global buffers between the ADC interrupt and the mainline

bit      bus_current_message_b;
uint16_t bus_current_max;
uint16_t bus_current_min;
uint32_t bus_current_sum;

bit      bus_voltage_message_b;
uint16_t bus_voltage_max;
uint16_t bus_voltage_min;
uint32_t bus_voltage_sum;

uint32_t charge_count;

// Global temporary message (MAINLINE ONLY)

CanMessage_t tempMsg;


//------------------------------------------------------------------------
// Local function declarations
//------------------------------------------------------------------------

void check_umbilical_voltage (void);


/*---------------------------------------------------------------------------\
| Function:     main
|
| Purpose:      Run all Mainline code
|
| Discussion:   Doesn't do much, huh?
|
| Expects:      Uh let's see... nothing.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void main (void)
{
    static uint16_t temp_int;

    // Initialize peripherals and interrupts

    Initialize();

    // Loop for ever...and......ev....er....and.......ev............er.

    tempMsg.header.all = 0b0010000010000000; //can_id_umbilical_voltage;
    tempMsg.header.bits.length = 8;
    tempMsg.data[0] = 0xFF;
    tempMsg.data[1] = 0xFF;
    tempMsg.data[2] = 0xFF;
    tempMsg.data[3] = 0xFF;
    tempMsg.data[4] = 0xFF;
    tempMsg.data[5] = 0xFF;
    tempMsg.data[6] = 0xFF;
    tempMsg.data[7] = 0xFF;
    tempMsg.data[8] = 0xFF;

    SendCanMessage(&tempMsg);

    while (TRUE)
    {
        // Check for received can messages and process them

        if(canMessageAvailable)
        {
            GetCanMessage(&tempMsg);
            Parse_Can_Message(&tempMsg);
        }

        // Check for new bus current measurement

        if(bus_current_message_b)
        {
            // WARNING WARNING WARNING: CHECK BIT FIELDS - 32BITS?
            temp_int = bus_current_sum / BUS_CURRENT_SAMPLE_COUNT;

            tempMsg.header.all = can_id_power_current;   // set can node header
            tempMsg.data[0] = bus_current_max >> 8; // max
            tempMsg.data[1] = bus_current_max;
            tempMsg.data[2] = temp_int >> 8;        // avg
            tempMsg.data[3] = temp_int;
            tempMsg.data[4] = bus_current_min >> 8; // min
            tempMsg.data[5] = bus_current_min;

            SendCanMessage(&tempMsg);
        }


        // Check for new bus voltage measurement

        if(bus_voltage_message_b)
        {
            // WARNING WARNING WARNING: CHECK BIT FIELDS - 32BITS?
            temp_int = bus_voltage_sum / BUS_VOLTAGE_SAMPLE_COUNT;

            tempMsg.header.all = can_id_power_voltage;   // set can node header
            tempMsg.data[0] = bus_voltage_max >> 8; // max
            tempMsg.data[1] = bus_voltage_max;
            tempMsg.data[2] = temp_int >> 8;        // avg
            tempMsg.data[3] = temp_int;
            tempMsg.data[4] = bus_voltage_min >> 8; // min
            tempMsg.data[5] = bus_voltage_min;

            SendCanMessage(&tempMsg);
        }

        // Check for new umbilical voltage measurement

        if(umbilical_voltage_ready_b)
        {
            check_umbilical_voltage();
        }
    }
}



/*---------------------------------------------------------------------------\
| Function:     Parse_Can_Message
|
| Purpose:      Interpet can message and act on it.
|
| Discussion:   see purpose
|
| Expects:      a can message with directions in it
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void Parse_Can_Message(CanMessage_t* receivedMsg)
{
	// GET REAL HEX VALUES FROM ANDREW


	// Can message to reset
	// implies dump to charge_count to EPROM ?
	if( ((*receivedMsg).data[0] == 0xAA)
		&& ((*receivedMsg).data[1] == 0xD5)
		&& ((*receivedMsg).data[2] == 0x23) )
	{
	   asm("reset");
	}

	// Can message to turn off IPS SWITCH
	// implies dump to charge_count to EPROM!
	if( ((*receivedMsg).data[0] == 0xff)
		&& ((*receivedMsg).data[1] == 0xff)
		&& ((*receivedMsg).data[2] == 0xff) )
	{
	   IPS_SWITCH = 0;
	}

	// Can message to turn on IPS SWITCH
	if( ((*receivedMsg).data[0] == 0xff)
		&& ((*receivedMsg).data[1] == 0xff)
		&& ((*receivedMsg).data[2] == 0xff) )
	{
	   IPS_SWITCH = 1;
	}

	// Assert Rocket Ready
	if( ((*receivedMsg).data[0] == 0xff)
		&& ((*receivedMsg).data[1] == 0xff)
		&& ((*receivedMsg).data[2] == 0xff) )
	{
		ROCKET_READY = 1;
	}

	// Return state of Rocet Ready
	if( ((*receivedMsg).data[0] == 0xff)
		&& ((*receivedMsg).data[1] == 0xff)
		&& ((*receivedMsg).data[2] == 0xff) )
	{
	   if(ROCKET_READY){
		 tempMsg.header.all = can_id_umbilical_voltage;
		 tempMsg.data[0] = (0xff);
   	   } else {
		 tempMsg.header.all = can_id_umbilical_voltage;
		 tempMsg.data[0] = (0xff);
	   }
       SendCanMessage(&tempMsg);
	}

} // end Parse_Can_Message





/*---------------------------------------------------------------------------\
| Function:     check_umbilical_voltage
|
| Purpose:      Check for a change in the umbilical cord voltage.
|
| Discussion:   Send a message when the umbilical voltage rises above or
|               falls below the cutoff for a certain length of time. We
|               want to debounce the voltage since it may be bouncy when
|               the umbilical is plugged in or yanked out. Also, if the
|               the voltage rises above the cutoff, then that's our signal
|               to turn on the avionics system so we set the state and
|               the IPS routine eventually turns on the avionics power.
|
|               See also the int0_application_handler; although it sets no
|               state it does wake the PIC up.
|
|               Since the umbilical voltage is checked at a very slow pace,
|               say 1 - 100Hz, we luckily don't have to worry about the ISR
|               changing the values in the umbilical_voltage variable while
|               we're checking it in this routine. Else we'd have to disable
|               the ADC interrupt which would be ugly.
|
| Expects:      For this routine to be called every time a new ADC
|               measurement of the umbilical voltage is done, and for the
|               umbilical voltage not to change while we're in this routine.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/02/2003 | Created.
\---------------------------------------------------------------------------*/

void check_umbilical_voltage (void)
{
    static uint16_t  umbilical_state_b_counter; // Can this be 8bits?

    // If the umbilical voltage has been low, then check to see if the
    // umbilical voltage is valid (above the cutoff). If it's above the
    // cutoff for the specified time (the debounce time) then switch the
    // state and set the avionics power state to 1 (on).

    if (umbilical_state_b == 0)
    {
        if (umbilical_voltage >= UMB_VOLTAGE_CUTOFF)
        {
            ++umbilical_state_b_counter;
            if (umbilical_state_b_counter > UMB_VOLTAGE_DEBOUNCE_COUNT)
            {
                umbilical_state_b = 1;
                umbilical_state_b_counter = 0;

                // Turn on the avionics system by setting the avionics on flag
                // (which will turn on the power in the IPS routines).

                avionics_power_state = 1;
            }
        }
        else
        {
            umbilical_state_b_counter = 0;
        }
    }

    // If the umbilical voltage has been high, then check to see if the
    // umbilical voltage is invalid (below the cutoff). If it's below the
    // cutoff for the specified time (the debounce time) then switch the
    // state.

    else
    {
        if (umbilical_voltage < UMB_VOLTAGE_CUTOFF)
        {
            ++umbilical_state_b_counter;
            if (umbilical_state_b_counter > UMB_VOLTAGE_DEBOUNCE_COUNT)
            {
                umbilical_state_b = 0;
                umbilical_state_b_counter = 0;

                // Note that we don't turn off the avionics system if
                // the umbilical voltage is low so we don't clear
                // avionics_power_state.
             }
        }
        else
        {
            umbilical_state_b_counter = 0;
        }
    }

    // In all cases, send the umbilical voltage message

    tempMsg.header.all = can_id_umbilical_voltage;
    tempMsg.data[0] = (0x00 | umbilical_state_b);
    tempMsg.data[1] = (umbilical_voltage >> 8);
    tempMsg.data[2] = umbilical_voltage;
    SendCanMessage(&tempMsg);
    umbilical_voltage_ready_b = 0;

}

